/**

SpagoBI - The Business Intelligence Free Platform

Copyright (C) 2005 Engineering Ingegneria Informatica S.p.A.

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spagobi.services.modules;

import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dispatching.module.AbstractModule;
import it.eng.spago.dispatching.service.list.basic.impl.DelegatedBasicListService; 
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.error.EMFUserError;
import it.eng.spagobi.bo.ModalitiesValue;
import it.eng.spagobi.bo.Parameter;
import it.eng.spagobi.bo.dao.DAOFactory;
import it.eng.spagobi.bo.dao.IParameterDAO;
import it.eng.spagobi.bo.dao.IParameterUseDAO;
import it.eng.spagobi.bo.dao.jdbc.DomainDAOImpl;
import it.eng.spagobi.bo.dao.jdbc.ParameterDAOImpl;
import it.eng.spagobi.constants.AdmintoolsConstants;
import it.eng.spagobi.constants.SpagoBIConstants;
import it.eng.spagobi.utilities.SpagoBITracer;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

/**
 * Implements a module which  handles all parameters management: has methods for parameters load 
 * detail, modify/insertion and deleting operations. The <code>service</code> method has  a 
 * switch for all these operations, differentiated the ones from the others by a <code>message</code> String.
 * 
 * @author sulis
 */
public class DetailParameterModule extends AbstractModule {
	private SourceBean parameter = null;
	private String modalita = "";
	private Boolean back = new Boolean(false);

	public void init(SourceBean config) {
	}

	/**
	 * Reads the operation asked by the user and calls the insertion, modify, detail and 
	 * deletion methods
	 * 
	 * @param request The Source Bean containing all request parameters
	 * @param response The Source Bean containing all response parameters
	 * @throws exception If an exception occurs
	 * 
	 */
	public void service(SourceBean request, SourceBean response) throws Exception {
		String message = (String) request.getAttribute("MESSAGEDET");
		SpagoBITracer.debug(AdmintoolsConstants.NAME_MODULE, "DetailParameterModule","service","begin of detail Engine modify/visualization service with message =" +message);

		EMFErrorHandler errorHandler = getErrorHandler();
		try {
			if (message == null) {
				EMFUserError userError = new EMFUserError(EMFErrorSeverity.ERROR, 101);
				SpagoBITracer.debug(AdmintoolsConstants.NAME_MODULE, "DetailParameterModule", "service", "The message parameter is null");
				throw userError;
			}
			if (message.trim().equalsIgnoreCase(AdmintoolsConstants.DETAIL_SELECT)) {
				String id = (String) request.getAttribute("id");
				getDetailParameter(id, response);
			} else if (message.trim().equalsIgnoreCase(AdmintoolsConstants.DETAIL_MOD)) {
				nameControl(request, "MODIFY");
				modDetailParameter(request, AdmintoolsConstants.DETAIL_MOD, response);
			} else if (message.trim().equalsIgnoreCase(AdmintoolsConstants.DETAIL_NEW)) {
				newDetailParameter(response);
			} else if (message.trim().equalsIgnoreCase(AdmintoolsConstants.DETAIL_INS)) {
				nameControl (request, "INSERT");
				modDetailParameter(request, AdmintoolsConstants.DETAIL_INS, response);
			} else if (message.trim().equalsIgnoreCase(AdmintoolsConstants.DETAIL_DEL)) {
				delDetailParameter(request, AdmintoolsConstants.DETAIL_DEL, response);
			}

		} catch (EMFUserError eex) {
			errorHandler.addError(eex);
			return;
		} catch (Exception ex) {
			EMFInternalError internalError = new EMFInternalError(EMFErrorSeverity.ERROR, ex);
			errorHandler.addError(internalError);
			return;
		}
	}


	
	/**
	 * Gets the detail of a parameter choosed by the user from the 
	 * parameters list. It reaches the key from the request and asks to the DB all detail
	 * parameter information, by calling the method <code>loadForDetailByParameterID</code>.
	 *   
	 * @param key The choosed engine id key
	 * @param response The response Source Bean
	 * @throws EMFUserError If an exception occurs
	 */
	
	private void getDetailParameter(String key, SourceBean response) throws EMFUserError {
		try {
			this.modalita = AdmintoolsConstants.DETAIL_MOD;
			response.setAttribute("modality", modalita);	
			Parameter parameter = DAOFactory.getParameterDAO().loadForDetailByParameterID(new Integer(key));
			response.setAttribute("parametersObj", parameter);		
			loadValuesDomain(response);	
		} catch (Exception ex) {
			SpagoBITracer.major(AdmintoolsConstants.NAME_MODULE, "DetailParameterModule","getDetailParameter","Cannot fill response container", ex  );
			throw new EMFUserError(EMFErrorSeverity.ERROR, 100);
		}
	}
	
	
	/**
	 * Inserts/Modifies the detail of a parameter according to the user request. 
	 * When a parameter is modified, the <code>modifyParameter</code> method is called; when a new
	 * parameter is added, the <code>insertParameter</code>method is called. These two cases are 
	 * differentiated by the <code>mod</code> String input value 
	 * 
	 * @param request The request information contained in a SourceBean Object
	 * @param mod A request string used to differentiate insert/modify operations
	 * @param response The response SourceBean 
	 * @throws EMFUserError If an exception occurs
	 * @throws SourceBeanException If a SourceBean exception occurs
	 */
	private void modDetailParameter(SourceBean request, String mod, SourceBean response)
		throws EMFUserError, SourceBeanException {
		try {
			String idStr = (String)request.getAttribute("id");
			Integer id = new Integer(idStr);
			String description = (String)request.getAttribute("description");
			String lengthStr = (String)request.getAttribute("length");
			Integer length = new Integer(lengthStr);
			String label = (String) request.getAttribute("label");
			String type = (String)request.getAttribute("type");
			String mask = (String)request.getAttribute("mask");
			String modality = (String)request.getAttribute("modality");
			String name = (String)request.getAttribute("name");

            Parameter parameter  = new Parameter();
            parameter.setId(id);
            parameter.setDescription(description);
			parameter.setLength(length);
			parameter.setLabel(label);
			parameter.setType(type);
			parameter.setMask(mask);
			parameter.setModality(modality);
			parameter.setName(name);
			
			if (mod.equalsIgnoreCase(AdmintoolsConstants.DETAIL_INS)) {
				DAOFactory.getParameterDAO().insertParameter(parameter);
			} else {
				DAOFactory.getParameterDAO().modifyParameter(parameter);
			}
            
		} catch (Exception ex) {			
			SpagoBITracer.major(AdmintoolsConstants.NAME_MODULE, "DetailParameterModule","modDetailParameter","Cannot fill response container", ex  );
			HashMap params = new HashMap();
			params.put(AdmintoolsConstants.PAGE, ListParametersModule.MODULE_PAGE);
			throw new EMFUserError(EMFErrorSeverity.ERROR, 1015, new Vector(), params);
			
			
			//HashMap errorParams = new HashMap();
			//errorParams.put(AdmintoolsConstants.PAGE, TreeObjectsModule.MODULE_PAGE);
			//errorParams.put(SpagoBIConstants.ACTOR, SpagoBIConstants.ADMIN_ACTOR);
			//errorParams.put(SpagoBIConstants.OPERATION, SpagoBIConstants.FUNCTIONALITIES_OPERATION);
			//EMFUserError error = new EMFUserError(EMFErrorSeverity.ERROR, 1007, new Vector(), errorParams);
			//getErrorHandler().addError(error);
			//return;
		}
		response.setAttribute("loopback", "true");
	}
	
	/**
	 * Deletes a parameter choosed by user from the parameters list.
	 * 
	 * @param request	The request SourceBean
	 * @param mod	A request string used to differentiate delete operation
	 * @param response	The response SourceBean
	 * @throws EMFUserError	If an Exception occurs
	 * @throws SourceBeanException If a SourceBean Exception occurs
	 */
	
	private void delDetailParameter(SourceBean request, String mod, SourceBean response)
		throws EMFUserError, SourceBeanException {
		try {
			IParameterDAO parDAO = DAOFactory.getParameterDAO();
			IParameterUseDAO parUseDAO = DAOFactory.getParameterUseDAO();
			String id = (String) request.getAttribute("id");
			//controls if the parameter has any use associated
			boolean hasUse = DAOFactory.getParameterUseDAO().hasParUseModes(id);
			if (hasUse){
					parUseDAO.eraseParameterUseByParId(Integer.valueOf(id));
					/*HashMap params = new HashMap();
					params.put(AdmintoolsConstants.PAGE, ListParametersModule.MODULE_PAGE);
					EMFUserError error = new EMFUserError(EMFErrorSeverity.ERROR, 1016, new Vector(), params);
					getErrorHandler().addError(error);
					return; */
					}
			//end of control
			//controls if the parameter has any object assosiated
			boolean hasObject = DAOFactory.getBIObjectParameterDAO().hasObjParameters(id);
			if (hasObject){
					HashMap params = new HashMap();
					params.put(AdmintoolsConstants.PAGE, ListParametersModule.MODULE_PAGE);
					EMFUserError error = new EMFUserError(EMFErrorSeverity.ERROR, 1017, new Vector(), params);
					getErrorHandler().addError(error);
					return;
					}
			
			Parameter parameter =  parDAO.loadForDetailByParameterID(new Integer(id)); 
			parDAO.eraseParameter(parameter);
			
		} catch (Exception ex) {
			SpagoBITracer.major(AdmintoolsConstants.NAME_MODULE, "DetailParameterModule","delDetailParameter","Cannot fill response container", ex  );
			throw new EMFUserError(EMFErrorSeverity.ERROR, 100);
		}
		response.setAttribute("loopback", "true");
	}
	
	/**
	 * Instantiates a new <code>Parameter<code> object when a new parameter insertion is required, in order
	 * to prepare the page for the insertion.
	 * 
	 * @param response The response SourceBean
	 * @throws EMFUserError If an Exception occurred
	 */
	
	private void newDetailParameter(SourceBean response) throws EMFUserError {
		try {
			this.modalita = AdmintoolsConstants.DETAIL_INS;
			response.setAttribute("modality", modalita);
            Parameter parameter = new Parameter();
			parameter.setId(new Integer(0));
			parameter.setDescription("");
			parameter.setLength(new Integer(0));
			parameter.setLabel("");
			parameter.setType("");
			parameter.setMask("");
			parameter.setTypeId(new Integer(0));
			parameter.setName("");
			loadValuesDomain(response);
			response.setAttribute("parametersObj", parameter);	
		} catch (Exception ex) {
			SpagoBITracer.major(AdmintoolsConstants.NAME_MODULE, "DetailParameterModule","newDetailParameter","Cannot prepare page for the insertion", ex  );
			throw new EMFUserError(EMFErrorSeverity.ERROR, 100);
		}
	}
	
	/**
	 * Loads all possible domain values which can be choosed for a parameter. Each of them
	 * is stored in a list of <code>Domain</code> objects put into response.
	 * When the isertion/modify parameters page is loaded, the user selects a domain value for the
	 * parameter by the Data Selection CD Check Button.
	 * 
	 * @param response The response SourceBean
	 * @throws EMFUserError If an Exception occurred
	 */
	private void loadValuesDomain(SourceBean response)  throws EMFUserError {
		  try {
			   DomainDAOImpl domainDao = new DomainDAOImpl ();
		       List list = new ArrayList();	
		       list =  DAOFactory.getDomainDAO().loadListDomainsByType("PAR_TYPE");
			   response.setAttribute ("listObj", list);
		  }
		  catch (Exception ex) {
			SpagoBITracer.major(AdmintoolsConstants.NAME_MODULE, "DetailParameterModule","loadValuesDomain","Cannot prepare page for the insertion", ex  );
			throw new EMFUserError(EMFErrorSeverity.ERROR, 100);
		  }
	}
	
	/**
	 * During a value constraint insertion/modify, controls if the name given to the parameter 
	 * is yet in use.
	 * 
	 * @param request	The request Source Bean
	 * @param operation Defines if the operation is of insertion or modify
	 * @throws EMFUserError If any Exception occurred
	 */
	private void nameControl (SourceBean request, String operation) throws EMFUserError {
		String label = (String)request.getAttribute("label");
		List allparameters = DAOFactory.getParameterDAO().loadAllParameters();
		if (operation.equalsIgnoreCase("INSERT")){
	   Iterator i = allparameters.iterator();
		while (i.hasNext()){
			Parameter parameter = (Parameter)i.next();
			String paramLabel = parameter.getLabel();
			if (paramLabel.equals(label)) {
				HashMap params = new HashMap();
				params.put(AdmintoolsConstants.PAGE, ListParametersModule.MODULE_PAGE);
				throw new EMFUserError(EMFErrorSeverity.ERROR, 1031, new Vector(), params);	
				}
			}
		}
		else{
			String currentId = (String)request.getAttribute("id");
			Iterator i = allparameters.iterator();
			while (i.hasNext()){
				Parameter parameter = (Parameter)i.next();
				String paramLabel = parameter.getLabel();
				String paramId = parameter.getId().toString();
				if (paramLabel.equals(label)&& (!currentId.equalsIgnoreCase(paramId))) {
					HashMap params = new HashMap();
					params.put(AdmintoolsConstants.PAGE, ListParametersModule.MODULE_PAGE);
					throw new EMFUserError(EMFErrorSeverity.ERROR, 1031, new Vector(), params);	
					}
				}
			
			
		}
	}	
	
}
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
//	private Hashtable getAddInfo(){
//		Vector addInfo = new Vector();
//		addInfo.add("Codice");
//		Hashtable information = new Hashtable();
//		information.put("121",addInfo);
//		return information;
//	}
//	
//	private Hashtable engineNameList(){
//		ConfigSingleton configSingleton = ConfigSingleton.getInstance();
//	    List engine = configSingleton.getAttributeAsList("ENGINES.ENGINE");
//	    Iterator engineIT = engine.iterator();
//	    String nameEngine = "";
//	    Hashtable nameHT = new Hashtable();
//	    while(engineIT.hasNext()){
//	    	SourceBean engineSB = (SourceBean)engineIT.next();
//	    	nameEngine = (String)engineSB.getAttribute("name");
//	    	nameHT.put(nameEngine, nameEngine);
//	    }
//	    return nameHT;
//	}

