/**

SpagoBI - The Business Intelligence Free Platform

Copyright (C) 2005 Engineering Ingegneria Informatica S.p.A.

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spagobi.services.modules;

import groovy.lang.Binding;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanAttribute;
import it.eng.spago.dispatching.module.list.basic.AbstractBasicListModule;
import it.eng.spago.paginator.basic.ListIFace;
import it.eng.spago.paginator.basic.PaginatorIFace;
import it.eng.spago.paginator.basic.impl.GenericList;
import it.eng.spago.paginator.basic.impl.GenericPaginator;
import it.eng.spago.security.IEngUserProfile;
import it.eng.spagobi.bo.ModalitiesValue;
import it.eng.spagobi.bo.ScriptDetail;
import it.eng.spagobi.bo.dao.DAOFactory;
import it.eng.spagobi.bo.dao.IModalitiesValueDAO;
import it.eng.spagobi.constants.ObjectsTreeConstants;
import it.eng.spagobi.constants.SpagoBIConstants;
import it.eng.spagobi.services.commons.LookupQueryDelegatedBasicListService;
import it.eng.spagobi.services.commons.LookupScriptDelegatedBasicListService;
import it.eng.spagobi.utilities.GeneralUtilities;
import it.eng.spagobi.utilities.PortletUtilities;
import it.eng.spagobi.utilities.SpagoBITracer;

import java.util.Iterator;
import java.util.List;

/**
 * Loads the predefined lookup list of values 
 * 
 * @author sulis
 */
public class ListLookupModalityValuesModule extends AbstractBasicListModule {
	
	
	/**
	 * Class Constructor
	 *
	 */
	public ListLookupModalityValuesModule() {
		super();
	} 
	
	
	
	/**
	 * Gets the list
	 * 
	 * @param request The request SourceBean
	 * @param response The response SourceBean
	 * @return ListIFace 
	 */
	public ListIFace getList(SourceBean request, SourceBean response) throws Exception {
		
		// laod the parameter use
		Integer idModVal = Integer.valueOf((String)request.getAttribute("mod_val_id"));
		IModalitiesValueDAO aModalitiesValueDAO = DAOFactory.getModalitiesValueDAO(); 
		ModalitiesValue modVal = aModalitiesValueDAO.loadModalitiesValueByID(idModVal); 

		// define the list of values
		ListIFace list = null;
		
		// get the input type from request (query || script)
		String inputType = (String)request.getAttribute("INPUT_TYPE");
		
		// different input type call different delegated class to build the list 
		if(inputType.equalsIgnoreCase(SpagoBIConstants.INPUT_TYPE_QUERY_CODE)) {
			String configQuery = modVal.getLovProvider();
			configQuery = PortletUtilities.cleanString(configQuery);
			SourceBean configModule = SourceBean.fromXMLString(configQuery);
			SourceBean moduleConfig = new SourceBean("MODULE_CONFIG");
			moduleConfig.setAttribute("TITLE", modVal.getDescription());
			moduleConfig.setAttribute("ORIGINAL_PAGE", request.getAttribute("ORIGINAL_PAGE"));
			moduleConfig.setAttribute("LOOKUP_PARAMETER_NAME", request.getAttribute("LOOKUP_PARAMETER_NAME"));
			String actor = (String)request.getAttribute(SpagoBIConstants.ACTOR);
			moduleConfig.setAttribute(SpagoBIConstants.ACTOR, actor);
			moduleConfig.setAttribute(configModule);
			response.setAttribute(SpagoBIConstants.INPUT_TYPE, inputType);
			response.setAttribute(moduleConfig);
			response.setAttribute("mod_val_id", idModVal);
			list = LookupQueryDelegatedBasicListService.getList(this, request, response, configModule);
		} else if(inputType.equalsIgnoreCase(SpagoBIConstants.INPUT_TYPE_SCRIPT_CODE)) {
			
			RequestContainer reqCont = getRequestContainer();
			SessionContainer session = reqCont.getSessionContainer();
			SessionContainer permSess = session.getPermanentContainer();
			IEngUserProfile profile = (IEngUserProfile)permSess.getAttribute(IEngUserProfile.ENG_USER_PROFILE);
			
			String lov = modVal.getLovProvider();
			ScriptDetail scriptDet = ScriptDetail.fromXML(lov);
			String script = scriptDet.getScript();
			Binding bind = GeneralUtilities.fillBinding(profile);
			String result = GeneralUtilities.testScript(script, bind);
			SourceBean resultSB = null;
			try{
				resultSB = SourceBean.fromXMLString(result);
			} catch(Exception e) {
				SpagoBITracer.major(ObjectsTreeConstants.NAME_MODULE, 
									"ListLookupModalityValuesModule", 
									"getList", "Error during parsing of the script result",e);
			}
			list = LookupScriptDelegatedBasicListService.getList(this, request, response, resultSB);
			List attrs =  resultSB.getContainedAttributes();
			Iterator iter = attrs.iterator();
			SourceBean configModule = new SourceBean("QUERY");
			while(iter.hasNext()) {
				SourceBeanAttribute sba = (SourceBeanAttribute)iter.next();
				SourceBean sb = (SourceBean)sba.getValue();
				configModule.setAttribute(sb);
			}

			SourceBean moduleConfig = new SourceBean("MODULE_CONFIG");
			moduleConfig.setAttribute("TITLE", modVal.getDescription());
			moduleConfig.setAttribute("ORIGINAL_PAGE", request.getAttribute("ORIGINAL_PAGE"));
			moduleConfig.setAttribute("LOOKUP_PARAMETER_NAME", request.getAttribute("LOOKUP_PARAMETER_NAME"));
			String actor = (String)request.getAttribute(SpagoBIConstants.ACTOR);
			moduleConfig.setAttribute(SpagoBIConstants.ACTOR, actor);
			moduleConfig.setAttribute(configModule);
			response.setAttribute(SpagoBIConstants.INPUT_TYPE, inputType);
			response.setAttribute(moduleConfig);
			response.setAttribute("mod_val_id", idModVal);
		}
		
		
		// filter the list 
		String valuefilter = (String)request.getAttribute(SpagoBIConstants.VALUE_FILTER);
		if((valuefilter!=null) && !(valuefilter.trim().equals(""))) {
			list = filterList(list, valuefilter, request);
		}
		
		return list;
		
	} 

	
	
	
	
	private ListIFace filterList(ListIFace list, String valuefilter, SourceBean request) {
		valuefilter = valuefilter.toUpperCase();
		String columnfilter = (String)request.getAttribute(SpagoBIConstants.COLUMN_FILTER);
		if((columnfilter==null)||(columnfilter.trim().equals(""))) {
			return list;
		}
		String typeFilter = (String)request.getAttribute(SpagoBIConstants.TYPE_FILTER);
		if((typeFilter==null)||(typeFilter.trim().equals(""))) {
			return list;
		}
		PaginatorIFace newPaginator = new GenericPaginator();
		newPaginator.setPageSize(list.getPaginator().getPageSize());
		SourceBean allrowsSB = list.getPaginator().getAll();
		List rows = allrowsSB.getAttributeAsList("ROW");
		Iterator iterRow = rows.iterator();
		while(iterRow.hasNext()) {
			SourceBean row = (SourceBean)iterRow.next();
			String value = (String)row.getAttribute(columnfilter);
			value = value.toUpperCase();
			if(typeFilter.equalsIgnoreCase(SpagoBIConstants.START_FILTER)) {
				if(value.startsWith(valuefilter))
					newPaginator.addRow(row);
			} else if(typeFilter.equalsIgnoreCase(SpagoBIConstants.END_FILTER)) {
				if(value.endsWith(valuefilter))
					newPaginator.addRow(row);
			} else if(typeFilter.equalsIgnoreCase(SpagoBIConstants.CONTAIN_FILTER)) {
				if(value.indexOf(valuefilter)!=-1)
					newPaginator.addRow(row);
			} else if(typeFilter.equalsIgnoreCase(SpagoBIConstants.EQUAL_FILTER)) {
				if(value.equals(valuefilter))
					newPaginator.addRow(row);
			}
		}
		ListIFace newList = new GenericList();
		newList.setPaginator(newPaginator);
		return newList;
	}
	
	
	
	
	
} 

