/**

SpagoBI - The Business Intelligence Free Platform

Copyright (C) 2005 Engineering Ingegneria Informatica S.p.A.

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spagobi.services.modules;

import javax.portlet.PortletPreferences;
import javax.portlet.PortletRequest;

import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dispatching.module.AbstractModule;
import it.eng.spago.security.IEngUserProfile;
import it.eng.spagobi.constants.ObjectsTreeConstants;
import it.eng.spagobi.constants.SpagoBIConstants;
import it.eng.spagobi.services.dao.TreeObjectsDAO;
import it.eng.spagobi.utilities.PortletUtilities;
import it.eng.spagobi.utilities.SpagoBITracer;

/**
 * Loads an objects hyerarchy (Tree) 
 * 
 * 
 * @author sulis
 */
public class TreeObjectsModule extends AbstractModule {

    public static final String MODULE_PAGE = "TreeObjectsPage";
    private static final String PATH_SYS_FUNCT = "SPAGOBI.CMS_PATHS.SYSTEM_FUNCTIONALITIES_PATH";
    private static final String MODALITY = "MODALITY";
    private static final String SINGLE_OBJECT = "SINGLE_OBJECT";
    private static final String FILTER_TREE = "FILTER_TREE";
    private static final String PATH_SINGLE_OBJECT = "PATH_SINGLE_OBJECT";
    private static final String PATH_SUBTREE = "PATH_SUBTREE";
    private static final String ROLE_SINGLE_OBJECT = "ROLE_SINGLE_OBJECT";
    private static final String HEIGHT_AREA = "HEIGHT_AREA";
	
    SessionContainer sessionContainer = null;
    
	public void init(SourceBean config) {
	}

	
	/**
	 * Loads tree and puts it into response, both with the execution actor
	 * 
	 * @param request The request Source Bean
	 * @param response The response Source Bean
	 * 
	 */
	public void service(SourceBean request, SourceBean response) throws Exception {
		try{	
			// get session
			RequestContainer requestContainer = getRequestContainer();
            sessionContainer = requestContainer.getSessionContainer();
            // get type of actor
			String actor = (String)request.getAttribute(SpagoBIConstants.ACTOR);
	        sessionContainer.setAttribute(SpagoBIConstants.ACTOR, actor);
            // get profile of the current user
            SessionContainer permanentSession = sessionContainer.getPermanentContainer();
            IEngUserProfile profile = (IEngUserProfile)permanentSession.getAttribute(IEngUserProfile.ENG_USER_PROFILE);
		    // get portlet preferences
            PortletRequest portReq = PortletUtilities.getPortletRequest();
			PortletPreferences prefs = portReq.getPreferences();
            // try to get the modality
            String modality = (String)prefs.getValue(MODALITY, "");
            // switch operation based on modality
            if(modality.equalsIgnoreCase(SINGLE_OBJECT)) {
            	singleObjectModalityHandler(request, response, prefs, actor);
            } else if(modality.equalsIgnoreCase(FILTER_TREE)) {
            	filterTreeModalityHandler(request, response);
            } else {
            	defaultModalityHandler(request, response, actor, profile);
            }
            
		} catch (Exception e) {
			SpagoBITracer.major(SpagoBIConstants.NAME_MODULE, 
								"TreeObjectsMOdule", 
								"service", 
								"Error while processing request of service",e);
			//TODO add a new EMFErrors to the stack of errors
		}
	}
	
	
	
	private void singleObjectModalityHandler(SourceBean request, SourceBean response,
			                                 PortletPreferences prefs, String actor) throws SourceBeanException {
		// get from preferences the path of the object and the execution role
		String path = prefs.getValue(PATH_SINGLE_OBJECT, "");
		String role = prefs.getValue(ROLE_SINGLE_OBJECT, "");
		// get from the preferences the area that shows the output
		String heightArea = prefs.getValue(HEIGHT_AREA, "");
        // if path is not set then throw an exception
		if(path.equals("")) {
        	
        }
		// if execution role is not set then throws an exception
        if(role.equals("")) {
        	
        }
        // set into the reponse the publisher name for object execution
        response.setAttribute(SpagoBIConstants.PUBLISHER_NAME, 
        					  SpagoBIConstants.PUBLISHER_LOOPBACK_SINGLE_OBJECT_EXEC);
        // set into the response the righr inforamtion for loopback
        response.setAttribute(SpagoBIConstants.ACTOR, actor);
        response.setAttribute(SpagoBIConstants.PATH, path);
        response.setAttribute(SpagoBIConstants.ROLE, role);
        // put in session the modality and the actor
        sessionContainer.setAttribute(SpagoBIConstants.MODALITY, 
        		                      SpagoBIConstants.SINGLE_OBJECT_EXECUTION_MODALITY);
        // if the height of the area in set put it into the session
        if(!heightArea.equals("")) 
        	sessionContainer.setAttribute(SpagoBIConstants.HEIGHT_OUTPUT_AREA, heightArea);
	}
	
	
	
	private void filterTreeModalityHandler(SourceBean request, SourceBean response) throws SourceBeanException {
        // put in session the modality and the actor
        sessionContainer.setAttribute(SpagoBIConstants.MODALITY, 
        		                      SpagoBIConstants.FILTER_TREE_MODALITY);
	}
	
	
	
	private void defaultModalityHandler(SourceBean request, SourceBean response, 
			                            String actor, IEngUserProfile profile) throws SourceBeanException {
		SourceBean dataResponseSysFunct = getFunctioanlitiesTree(profile);
		response.setAttribute(dataResponseSysFunct);
		response.setAttribute(SpagoBIConstants.ACTOR, actor);
		response.setAttribute(SpagoBIConstants.MODALITY, SpagoBIConstants.ENTIRE_TREE_MODALITY);
		Object objview = request.getAttribute(SpagoBIConstants.OBJECTS_VIEW);
		if(objview!=null) {
			response.setAttribute(SpagoBIConstants.OBJECTS_VIEW, objview);
		}
		Object listPage = request.getAttribute(SpagoBIConstants.LIST_PAGE);
		if(listPage!=null) {
			response.setAttribute(SpagoBIConstants.LIST_PAGE, listPage);
		}
        // put in session the modality and the actor
        sessionContainer.setAttribute(SpagoBIConstants.MODALITY, 
        		                      SpagoBIConstants.ENTIRE_TREE_MODALITY);
	}
	
	
	
	
	private SourceBean getFunctioanlitiesTree(IEngUserProfile profile) {
		SourceBean pathSysFunctSB = (SourceBean)ConfigSingleton.getInstance().getAttribute(PATH_SYS_FUNCT);
	    String pathSysFunct = pathSysFunctSB.getCharacters();
        TreeObjectsDAO objDao = new TreeObjectsDAO();
        SourceBean dataResponseSysFunct = objDao.getXmlTreeObjects(pathSysFunct, profile);
        return dataResponseSysFunct;
	}
	
	
}	
	
	