package it.eng.spago.cms.exec;

import it.eng.javax.jcr.Node;
import it.eng.javax.jcr.Property;
import it.eng.javax.jcr.PropertyIterator;
import it.eng.javax.jcr.RepositoryException;
import it.eng.javax.jcr.Session;
import it.eng.javax.jcr.Value;
import it.eng.javax.jcr.ValueFactory;
import it.eng.spago.base.SourceBean;
import it.eng.spago.cms.access.PermissionVerifier;
import it.eng.spago.cms.access.PermissionVerifierManager;
import it.eng.spago.cms.exceptions.OperationExecutionException;
import it.eng.spago.cms.exceptions.PathNotValidException;
import it.eng.spago.cms.exceptions.ValueCastException;
import it.eng.spago.cms.exec.entities.ElementProperty;
import it.eng.spago.cms.util.OperationDescriptorUtils;
import it.eng.spago.cms.util.Path;
import it.eng.spago.cms.util.constants.CMSConstants;
import it.eng.spago.cms.util.constants.OperationsConstants;

import java.io.InputStream;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;



/**
 * 
 * Implements method that perform the set operation
 * on the repository
 * 
 */
public class SetObjectImpl {

	
	/**
	 * Implements the set operation on the repository
	 * 
	 * @param parameters, SourceBean of operation configuration
	 * @param conn, Connection to the repository
	 * @param roles, Array of user roles string names 
	 * @return SourceBean that contains information about execution
	 * @throws OperationExecutionException, if some error occurs during
	 * the execution 
	 */
	protected SourceBean execute(SourceBean parameters, CMSConnection conn, String [] roles) throws OperationExecutionException{
		
		Session _session = conn.getSession();
		String prefixSys = conn.getPrefixPropertySystem();
		String prefixUsr = conn.getPrefixPropertyUser();
		String nodeName;
		String pathStr;
		String pathFatherStr;
		Path path;
		Path pathFather;
		String type;
		Node node = null;
		Node nodeFather = null;
		Node rootNode = null;
		InputStream data = null;
		Hashtable properties = new Hashtable();
		boolean nodeExist;
		boolean nodeFatherExist;
		boolean cancProps = false;
		
		
		try{ 
			
			// ********************************************************************************
			// SET VARIABLE
			// ********************************************************************************
						
            // get root node
			rootNode = _session.getRootNode();
			// get path string new object
		    pathStr = OperationDescriptorUtils.getStringAttribute(parameters, OperationsConstants.PATH);
		    // get flag cancel prop
		    Boolean cancPropB = OperationDescriptorUtils.getBooleanAttribute(parameters, OperationsConstants.OLDPROPERTIES);
		    if(cancPropB != null) {
		    	cancProps = cancPropB.booleanValue();
		    }
		    // get type object
		    type = OperationDescriptorUtils.getStringAttribute(parameters, CMSConstants.TYPE);
		    // if object is type content get stream data
		    if(type.equalsIgnoreCase(OperationsConstants.CONTENT)) {
		    	data = (InputStream)OperationDescriptorUtils.getObjectAttribute(parameters, OperationsConstants.STREAM);
		    }
		    // get path object
		    path = Path.create(pathStr);
		    // cotrol if path is root
		    if(path.isRootNode()) {
		    	throw new OperationExecutionException("SetObjectImpl:execute: it's not possible to " +
		    									      "execute set operation on root node");
		    } else {
		    	 // get name node
			    nodeName = path.getNameLastElement();
		    }
		    // get path father
		    pathFather = path.getPathAncestor(1); 
		    // get properties
		    properties = OperationDescriptorUtils.getProperties(parameters);
		    //  control if node exists
		    try {
		    	node = rootNode.getNode(path.getRootRelativePathStr());
		    	nodeExist = true;
		    } catch (Exception e) {
		    	nodeExist = false;
		    }
		    // control if father node exists
		    try {
		    	if(pathFather.isRootNode()) {
		    		nodeFather = rootNode;
		    	} else {
		    		nodeFather = rootNode.getNode(pathFather.getRootRelativePathStr());
		    	}
		    	nodeFatherExist = true;
		    } catch (Exception e) {
		    	nodeFatherExist = false;
		    }
		    
      
		    
		    // ********************************************************************
		    // CONTROLS ON INPUT PARAMS
		    // ********************************************************************
		    
		    
		    if(type.equals(CMSConstants.CONTAINER) && (data != null)) {
		    	throw new OperationExecutionException("SetObjectImpl::execute: a container can have properties " +
		    				                          "but not content bytes");
		    }
		    if(nodeExist) {
		    	String setType = node.getProperty(prefixSys+":"+OperationsConstants.TYPE).getValue().getString();
		    	if(!setType.equals(type)) {
		    		throw new OperationExecutionException("SetObjectImpl::execute: node already exists but types " +
		    							                  "are not correspondent ");
		    	}
		    }
		    
		    
		    
		    // ********************************************************************
		    // CONTROLS PERMISSION
		    // ********************************************************************
		    
			PermissionVerifier verifier = null;
			try {
				verifier = PermissionVerifierManager.getPermissionVerifier(conn.getNameRepositoryPool());
			} catch (Exception e) {
				throw new OperationExecutionException("GetObjectImpl::execute: error during the " +
		                "PermissionVerifier recover", e);
			}
		    if(!verifier.canAddNode(pathStr, roles)) {
		    	throw new OperationExecutionException("SetObjectImpl::execute: permission to add node " +
		                "into this path denied");
		    }
		    if(cancProps || !properties.isEmpty() ) {
			    if(!verifier.canSetProperty(pathStr, roles)) {
			    	throw new OperationExecutionException("SetObjectImpl::execute: permission to set or erase properties " +
	                "on this path denied");
			    }
		    }
		    
	    
		    // ********************************************************************
		    // IF CHAIN OF PATH FATHERS DOESN'T EXIST IT'S NECESSARY TO CREATE IT
		    // ********************************************************************
		        
		    if(!nodeFatherExist) {
		    	throw new OperationExecutionException("SetObjectImpl::execute: node father doesn't exist ");
		    	//TODO if the chain of father doesn't exist it's necessary to create it
		    	// simply control the commented method below
		    	//nodeFather = createPath(conn, path);
		    }
		    
				        
		    // **********************************************************************
		    // SAVE AND/OR CHECKIN
		    // **********************************************************************
		   
		    if(!nodeExist){
		    	node = nodeFather.addNode(nodeName, "nt:unstructured");
			    node.addMixin("mix:versionable");
			}
			else {
				if(!node.isCheckedOut()) {
				   	node.checkout();
				}
			}
		
			// set type and eventual content
			node.setProperty(prefixSys + ":"+OperationsConstants.TYPE, type);
		    if(type.equalsIgnoreCase(CMSConstants.CONTENT)) {
		        node.setProperty(prefixSys+":"+OperationsConstants.STREAM, data);
		      }
		    
		    // if cancel old properties it's necessary to erase all user property of the node
		    if(cancProps) {
		    	PropertyIterator propIter = node.getProperties();
		    	while(propIter.hasNext()) {
		    		Property prop = propIter.nextProperty();
		    		String nameP = prop.getName();
		    		if(nameP.startsWith(prefixUsr) ) {
		    			prop.remove();
		    		}
		    	}
		    	if(nodeExist) {
		    		node.save();
		    	}
		    }
		    
		    // set new properties
		    Enumeration propEnum = properties.keys();
		    ValueFactory vf = _session.getValueFactory();
		    while(propEnum.hasMoreElements()){
		    	String nameProp = (String)propEnum.nextElement();
		    	ElementProperty propObj = (ElementProperty)properties.get(nameProp);
		    	Value[] jcrvalues = propObj.getJCRValues(vf);
		    	if(jcrvalues.length == 0) {
                	continue;
                }
                nameProp = prefixUsr + ":" + nameProp;
                node.setProperty(nameProp, jcrvalues);
		    }
		    
		    // save
		    if(nodeExist) {
		        node.save();
		    	node.checkin();
		    	node.checkout();
		    }
		    else {
		      //_session.save();
		      nodeFather.save();
		      node.checkin();
		      node.checkout();
		    }
	        
	        
		}
		catch(PathNotValidException mpe){
			throw new OperationExecutionException("SetObjectImpl::execute: malformed path", mpe);
		}
		catch(RepositoryException re){
			throw new OperationExecutionException("SetObjectImpl::execute: repository acess error", re);
		}
		catch(ValueCastException vce) {
			throw new OperationExecutionException("SetObjectImpl::execute: property value cast exception", vce);
		}
		return null;
	}
	
	
	
	
	
	
	
	

	
	
	
	
	private Node createPath(CMSConnection conn, Path path) throws OperationExecutionException {
        
		Session session = conn.getSession();
		String prefix = conn.getPrefixPropertySystem();
        Path pathElement = path;
        Path pathTmp = null;
		Node root = null;
		Node firstNodeExist = null;
		String firstNodeExistStr = null;
		Vector elementToAdd = new Vector();
		
		try {
			root = session.getRootNode();
			firstNodeExist = root;
			firstNodeExistStr = "/";
            // trovo il primo nodo esistente 
		    Node nodeFind;
		    String pathFindStr;
		    pathTmp = path.getPathAncestor(1);
		    while(!pathTmp.isRootNode()) {
				pathFindStr = pathTmp.getRootRelativePathStr();
				try {
					nodeFind = root.getNode(pathFindStr);
				}
				catch(Exception e) {
					String nameElementToCreate = pathTmp.getNameLastElement();
					elementToAdd.add(0, nameElementToCreate);
					pathTmp = pathTmp.getPathAncestor(1);
					continue;
				}
                // if the node exist exit from while
				firstNodeExist = nodeFind;
				firstNodeExistStr = "/" + pathFindStr;
				break;
			}		    
			// checkout node
			if(firstNodeExist.isNodeType("mix:versionable")) {
				if(!firstNodeExist.isCheckedOut()) {
					firstNodeExist.checkout();
				}
			}
			// create the chain of paths 
			for(int i=0; i<elementToAdd.size(); i++) {
				String newElement = (String)elementToAdd.get(i);
				Node tmpnode = firstNodeExist;
				firstNodeExist = firstNodeExist.addNode(newElement);
				firstNodeExist.addMixin("mix:versionable");
				firstNodeExist.setProperty(prefix + ":"+CMSConstants.TYPE, CMSConstants.CONTAINER);
				tmpnode.save();
				firstNodeExist.checkin();
				firstNodeExist.checkout();
			}
			// return the last node created that will be the father of the node to create
			return firstNodeExist;
			
			
		}
		catch(Exception e) {
			throw new OperationExecutionException("SetObjectImpl::createPath:Unable to create path");
		}
	}
	
	
}
