package it.eng.spago.navigation;

import it.eng.spago.base.SourceBean;
import it.eng.spagobi.constants.ObjectsTreeConstants;
import it.eng.spagobi.utilities.SpagoBITracer;

import javax.portlet.PortletRequest;
import javax.portlet.PortletSession;

public class SbiNavigationManager {

    public static final String SBI_NAVIGATOR_RESET = "SBI_NAVIGATOR_RESET";
    public static final String SBI_NAVIGATOR_DISABLED = "SBI_NAVIGATOR_DISABLED";
    public static final String SBI_NAVIGATOR_BACK_TO = "SBI_NAVIGATOR_BACK_TO";
    public static final String SBI_NAVIGATOR_BACK_TO_MARK = "SBI_NAVIGATOR_BACK_TO_MARK";
    public static final String SBI_NAVIGATOR_MARK = "SBI_NAVIGATOR_MARK";
	
	public static SourceBean controlNavigation (PortletRequest portletRequest, SourceBean serviceRequest) {
		
		SpagoBITracer.debug(ObjectsTreeConstants.NAME_MODULE,
				"NavigationManager",
				"controlNavigation",
				"Method invoked.");
        
        // If navigator is disabled return the serviceRequest unmodified
		String navigatorDisabled = (String) serviceRequest.getAttribute(SBI_NAVIGATOR_DISABLED);
		if ("true".equalsIgnoreCase(navigatorDisabled)) {
			SpagoBITracer.debug(ObjectsTreeConstants.NAME_MODULE, 
					"NavigationManager",
					"controlNavigation",
					"Navigator disabled; return from the method without any modifications to the request.");
			return serviceRequest;
		}
		
		// Makes a clone of the serviceRequest, to be returned in case of errors
		SourceBean savedServiceRequest = (SourceBean) serviceRequest.cloneObject();
		
		try {
			
			// Retrieves the navigator object from session; if it does not exist it creates a new one
			SbiNavigator navigator = retrieveNavigatorFromSession(portletRequest);
			
			String navigatorBackTo = (String) serviceRequest.getAttribute(SBI_NAVIGATOR_BACK_TO);
			if (navigatorBackTo != null) {
				int position = Integer.parseInt(navigatorBackTo);
				MarkedRequest markedRequest = navigator.goBackToPosition(position);
				SpagoBITracer.debug(ObjectsTreeConstants.NAME_MODULE, 
						"NavigationManager",
						"controlNavigation",
						"NAVIGATOR_BACK_TO parameter found; return request at position " + position + " of the stack.");
				return markedRequest.getRequest();
			}
			
			String navigatorBackToMark = (String) serviceRequest.getAttribute(SBI_NAVIGATOR_BACK_TO_MARK);
			if (navigatorBackToMark != null) {
				MarkedRequest markedRequest = navigator.goBackToMark(navigatorBackToMark);
				SpagoBITracer.debug(ObjectsTreeConstants.NAME_MODULE, 
						"NavigationManager",
						"controlNavigation",
						"NAVIGATOR_BACK_TO_MARK parameter found; return request with mark " + navigatorBackToMark + " of the stack.");
				return markedRequest.getRequest();
			}
			
			String navigatorReset = (String) serviceRequest.getAttribute(SBI_NAVIGATOR_RESET);
			if (navigatorReset != null) {
				navigator.reset();
				SpagoBITracer.debug(ObjectsTreeConstants.NAME_MODULE, 
						"NavigationManager",
						"controlNavigation",
						"NAVIGATOR_RESET parameter found; stack reset executed.");
				return serviceRequest;
			}
			
			String mark = (String) serviceRequest.getAttribute(SBI_NAVIGATOR_MARK);
			MarkedRequest markedRequest = new MarkedRequest(serviceRequest, mark);
			navigator.add(markedRequest);
			SpagoBITracer.debug(ObjectsTreeConstants.NAME_MODULE, 
					"NavigationManager",
					"controlNavigation",
					"Request put in the stack " + (mark == null ? "without mark." : "with mark '" + mark + "'. Returning unmodified request."));
			return serviceRequest;
			
		} catch (Exception e) {
			SpagoBITracer.major(ObjectsTreeConstants.NAME_MODULE, 
					"NavigationManager",
					"controlNavigation",
					"ERROR: " + e.getMessage() + ". Returning the original request.",
					e);
			return savedServiceRequest;
		}

	}
	
	// Retrieves the navigator object from session; if it does not exist it creates a new one
	private static SbiNavigator retrieveNavigatorFromSession (PortletRequest portletRequest) {
		PortletSession session = portletRequest.getPortletSession();
		Object navigatorObj = session.getAttribute("SBI_NAVIGATOR");
		SbiNavigator navigator = null;
		if (navigatorObj != null) navigator = (SbiNavigator) navigatorObj;
		else {
			navigator = new SbiNavigator();
			session.setAttribute("SBI_NAVIGATOR", navigator);
		}
		return navigator;
	}
	
}
